%% Part1: simulate the model to generate data

clear;
clc;


global M;
M = 4.5;          % control parameter
dt = 0.1;         % simulation time step
sampling_rate=10;  % 1 sample every sampling_rate*dt is measured
nPeriods =1000/dt; % total simulation steps



Diffusion = 0.01*[.2 0 0 0;0 0.5 0 0;0 0 0.8 0;0 0 0 1]; % random Diffusion matrix, ensure positive solution
% Diffusion = 0.01*[1 0 0 0;0 1 0 0;0 0 1 0;0 0 0 1]; % random Diffusion matrix, ensure positive solution

F = @(t, x) lotka_volterra(t,x);
G = @(t, x) Diffusion;
x0=[0.0067;21.5805;0.0496;25.8178]; % initial condition; set to be at equilibrium for M

SDE = sde(F,G,'StartState', x0);
[S,T] = simulate(SDE, nPeriods, 'DeltaTime', dt);
S_detrend=detrend(S);
Sn=S_detrend(1:sampling_rate:end,:);  % to control sampling rate. 10 

figure;subplot(2,2,1);plot(T,S(:,1));xlabel('Time');ylabel('Species1 population')
       subplot(2,2,2);plot(T,S(:,2));xlabel('Time');ylabel('Species2 population')
       subplot(2,2,3);plot(T,S(:,3));xlabel('Time');ylabel('Species3 population')
       subplot(2,2,4);plot(T,S(:,4));xlabel('Time');ylabel('Species4 population')


%% Part2: compute free decays using NExT Technique


ref_ch=[2];     % selected reference channel(s)
window_length=100/(dt*sampling_rate);   %number of data points in each window
overlap=0.75;                           %percentage of overlap between windows
N=length((1-overlap)*window_length:(1-overlap)*window_length:length(Sn)-window_length)-4;   % number of possible window shifts              

FD_time= NExT(Sn',ref_ch,window_length,N,overlap,window_length/2); % Approximate the free decays
Rdecays=FD_time(:,1:30);                % not all part of the obtained signal is required to be used in the identification                                         


%plot the free decays for reference channel 2
figure;subplot(2,2,1);plot(FD_time(1,2:length(ref_ch):end)/FD_time(1,2))
       subplot(2,2,2);plot(FD_time(2,2:length(ref_ch):end)/FD_time(2,2))
       subplot(2,2,3);plot(FD_time(3,2:length(ref_ch):end)/FD_time(3,2))
       subplot(2,2,4);plot(FD_time(4,2:length(ref_ch):end)/FD_time(4,2))


%% Part 3: modal identification using ERA and the approximated free decays
numref=length(ref_ch);
fs=1/(sampling_rate*dt);          %sampling frequency
cutoff=4;                            % number of active modes. a plot of singular calues will be shown. The number of modes should be chosen based on this plot.
ncols=round(2/3*length(Rdecays)); % number of columns to construct Hankel matrix (~ more than 2/3 of No. of data)
nrows= 20;                       % number of rows, 10 to 20 * number of modes
inputs=length(ref_ch);              % number of repeated experiments

Result_ERA=ERA(Rdecays,fs,ncols,nrows,inputs,cutoff)

Eigenvalues=Result_ERA.Evalues
Evectors=Result_ERA.Evectors










