% function [ S, T ] = harvesting_model( model, size,  C_pattern, bifurcation_parameter, sigma, spatial_parameter, K, IC, nPeriods, dt,Noise_intensity)
function [ S, T ] = harvesting_model(bifurcation_parameter, nPeriods, dt,Noise_intensity)

params=parameters;
r=params.r;  % spatial parameter
K=params.K;
C_pattern=params.C_pattern;
x_ini=params.IC; %initial condition for c0=2;
shape = size(r);
size_r = shape(1);
c0 = bifurcation_parameter;  % bifurcation parameters



 % stochastic simulation
    
    % diffusion
    G0 = eye(size_r*size_r, size_r*size_r);
    G = @(t,X) Noise_intensity*G0;

    SDE = sde(@ harvest,G,'StartState', x_ini);
    [S,T] = simulate(SDE, nPeriods, 'DeltaTime', dt);

    function dpop = harvest(t, pop)
    % harvesting model
        d_r = 0.2;
        c = (c0)*C_pattern ;

        % reshape
        data = zeros(size_r, size_r);
        for j = 1:size_r
            data(j,:) = pop((j-1)*size_r+1:j*size_r);
        end

        % define dpop with reflective boundary condition
        dpop = zeros(size_r*size_r,1);
        for j=1:size_r
            for k=1:size_r
                X = data(j,k);
                dpop((j-1)*size_r+k) = r(j,k)*X*(1-X/K(j,k))-c(j,k)*X^2/(X^2+1)+d_r*diffusion(size_r, data, j, k);     
            end
        end    
    end


end

function diff = diffusion(size, data, j, k)
    % calculate diffusion from neighboring cells
    X = data(j,k);
    if j+1>size
        X1 = X;
    else 
        X1 = data(j+1,k);
    end

    if j-1<1
        X2 = X;
    else 
        X2 = data(j-1,k);
    end

    if k-1<1
        X3 = X;
    else 
        X3 = data(j,k-1);
    end

    if k+1>size
        X4 = X;
    else 
        X4 = data(j,k+1);
    end
    diff = X1+X2+X3+X4-4*X;
end