%Simulation for the 5 by 5 spatial harvesting model
clear
clc


dt=0.1;
sampling_rate=1;
c0=2;            % control parameter (harvesting rate)
Noise_intensity=0.05;
nPeriods=10000;

[S, T] = harvesting_model(c0, nPeriods, dt, Noise_intensity);
% Subset=[1:25]; % The cell indexes which are being measured
Subset=[1,6,11,16,21,22,23,24,25]; % Example: when only the cells on the edge are measured
S_subset=S(:,Subset);
S_detrend=S_subset-mean(S_subset,1); % detrend to transfer the mean velue to zero
Sn=S_detrend(1:sampling_rate:end,:); 


%% Part2: compute free decays using cross correlations (NExT)

ref_ch=[5];  
window_length=100/(dt*sampling_rate);   %number of data points in each window
overlap=0.75;         %percentage of overlap between windows
N=length((1-overlap)*window_length:(1-overlap)*window_length:length(Sn)-window_length); % number of possible window shifts                %total number of windows

FD_time= NExT(Sn',ref_ch,window_length,N,overlap,window_length/2); % Approximate the free decays
Rdecays=FD_time(:,1:100);                % not all part of the obtained signal is required to be used in the identification                                         


%Sample plot: free decays for reference channel 21 and cell number 16
figure;plot(FD_time(4,5:length(ref_ch):end)/FD_time(4,5))




%% Part 3: modal identification using ERA
clear Eigenvalues Evectors Result_ERA
numref=length(ref_ch);
fs=1/(sampling_rate*dt);          %sampling frequency
cutoff=6;                        % number of active modes. a plot of singular calues will be shown. The number of modes should be chosen based on this plot.
ncols=round(2/3*length(Rdecays)); % number of columns in Hankel matrix,
nrows=25;                         % number of rows in Hankel matrix,
inputs=length(ref_ch);            % number of repeated experiments

Result_ERA=ERA(Rdecays,fs,ncols,nrows,inputs,cutoff);
Eigenvalues=Result_ERA.Evalues     % Identified eigenvalues
Evectors=Result_ERA.Evectors       % Identified eigenvectors



ij=1;
for k = 1:length(Eigenvalues)
      if isreal(Eigenvalues(k))==1
          REigenvalues(ij)=Eigenvalues(k);
          REvectors(:,ij)=Evectors(:,k);
          ij=ij+1
      end
end

[Sort_evalues,ms]=sort((REigenvalues),'descend')
Dominant_mode=abs(REvectors(:,ms(1)))/norm(REvectors(:,ms(1)));
figure;plot(Dominant_mode)




